import cocos
import pymunk
import pyglet
import math


from cocos.scenes.transitions import *
from cocos.actions import *

from pyglet.window import key, mouse
from pymunk.vec2d import Vec2d
import Context
import Actor.Blob
import GameUI
import LevelEditorUI

class MainScene( cocos.scene.Scene ):

    def __init__( self, levelClass ):
        super( MainScene, self ).__init__()
        pymunk.init_pymunk()

        self.space = pymunk.Space()
        self.space.gravity = (0,-500)


        self.context = Context.Context()
        self.context.space = self.space


        self.levelClass = levelClass
        self.context.level = levelClass(self.context)
        self.add(self.context.level)

        self.gameui =  GameUI.GameUI(self.context)
        self.add(self.gameui)

        #self.add( LevelEditorUI.LevelEditorUI(self.context))

        self.levelindex = 0
        self.levelorder = ["level1", "level2", "level3", "level4","level5", "level6", "level7"]
        #self.levelorder = ["level2", "level7"]


        self.logosprite = cocos.sprite.Sprite("bouncy2logo.png")
        self.helpsprite = cocos.sprite.Sprite("help.png")
        self.logosprite.x, self.logosprite.y = 600, 500
        self.helpsprite.x, self.helpsprite.y = 500, 300

        #self.state = "WaitingToStart"
        self.state = "Logo"
        self.startlabel = pyglet.text.Label('Click the mouse to start!',
                          font_name='Times New Roman',
                          font_size=40,
                          x=100, y=200)

        self.finishlabel = pyglet.text.Label('You completed the game!',
                          font_name='Times New Roman',
                          font_size=40,
                          x=200, y=300)

        self.label = pyglet.text.Label('Bouncy 2',
                          font_name='Times New Roman',
                          font_size=100,
                          x=400, y=500)

        tutlabels = [
        "Get the bouncies to the goal.",
        "Change their ability by right clicking on them.",
        "Experiment to figure out how they work!",
        "You'll lose if you use up all of the bouncies."]

        self.labeltuts = []
        tuty = 480
        for label in tutlabels:
            tuty -= 30
            self.labeltuts.append( pyglet.text.Label(label,
                              font_name='Times New Roman',
                              font_size=24,
                              x=310,
                              y=tuty))




        self.labelwon = pyglet.text.Label('Level complete!',
                          font_name='Times New Roman',
                          font_size=50,
                          x=300, y=300)

        self.labellost = pyglet.text.Label('Level failed!',
                          font_name='Times New Roman',
                          font_size=50,
                          x=300, y=300)

        pyglet.gl.glEnable(pyglet.gl.GL_LINE_SMOOTH)
        pyglet.gl.glLineWidth(2)
        pyglet.gl.glPointSize(2)


    def update( self ):

        if self.state == "WaitingToStart" or self.state == "WaitingToStartFromLose" or self.state == "WaitingToStartFromWon":
            if self.gameui.mouseClicked:
                self.context.level.load(self.context, self.levelorder[self.levelindex] + ".txt")
                self.state = "Playing"
                self.label.visible = False

        if self.state == "GameFinished":
            if self.gameui.mouseClicked:
                self.context.level.load(self.context, self.levelorder[self.levelindex] + ".txt")

        if self.state == "Logo":
            if self.gameui.mouseClicked:
                self.state = "Help"
                self.gameui.mouseClicked = False
                return

        if self.state == "Help":
            if self.gameui.mouseClicked:
                self.state = "WaitingToStart"
                self.context.level.load(self.context, "level8.txt")
                self.gameui.mouseClicked = False
                return

        self.gameui.mouseClicked = False
        dt = 1/60.
       # if self.state != "Playing:":
        #    dt = 1/600

        if self.gameui.hovered is not None:
            self.space.step(dt * 0.2)
        else:
            for i in range(5):
                self.space.step(dt * 0.2)


        self.context.dt = dt
        self.context.level.update(self.context)

        if self.state == "Playing":
            if self.context.level.isWon():
                if self.levelindex == len(self.levelorder) - 2:
                    self.state = "GameFinished"
                    self.levelindex += 1
                else:
                    self.state = "WaitingToStartFromWon"
                    self.levelindex += 1

            if self.context.level.isLost():
                print "Lost"
                self.state = "WaitingToStartFromLose"

    def draw( self ):
        if self.state == "Logo":
            self.logosprite.draw()
            return

        if self.state == "Help":
            self.helpsprite.draw()
            return

        if self.state == "GameFinished":
           # self.label.draw()
            self.finishlabel.draw()

        if self.state != "Playing":
            #self.tutbg.draw()
            #self.label.draw()
            self.startlabel.draw()

            for label in self.labeltuts:
                label.draw()

            if self.state == "WaitingToStartFromLose":
                self.labellost.draw()

            if self.state == "WaitingToStartFromWon":
                self.labelwon.draw()


        debug_draw = 0
        if not debug_draw:
            return

        pyglet.gl.glPushMatrix()
        self.transform()

        pyglet.gl.glLineWidth(1)
        space = self.space
        for s in space.shapes + space.static_shapes:
            if isinstance(s, pymunk.Segment):
                p1 = s.body.position + s.a.rotated(s.body.angle)
                p2 = s.body.position + s.b.rotated(s.body.angle)
                c = (1, 1, 1)
                if hasattr(s, 'color'):
                    c = s.color
                v2f = sum(map(tuple, [p1,p2]), ())
                pyglet.graphics.draw(2, pyglet.gl.GL_LINES,
                                     ('v2f', v2f),
                                      ('c3f', c * 2))
            elif isinstance(s, pymunk.Circle):
                c = (1, 1, 1)
                if hasattr(s, 'color'):
                    c = s.color
                draw_circle(s.body.position, s.radius, color = c, steps = 18, filled = False)


        pyglet.gl.glLineWidth(5)
        for c in space.constraints:
            if isinstance(c, pymunk.PinJoint):
                p1 = c.a.position + c.anchr1.rotated(c.a.angle)
                p2 = c.b.position + c.anchr2.rotated(c.b.angle)
                pyglet.graphics.draw(2, pyglet.gl.GL_POINTS,
                                     ('v2f', (p1.x, p1.y, p2.x, p2.y)),
                                      ('c3f', (0.0, 0.0, 1.0, 1,1,0)))
            elif isinstance(c, pymunk.DampedSpring):
                p1 = c.a.position + c.anchr1.rotated(c.a.angle)
                p2 = c.b.position + c.anchr2.rotated(c.b.angle)
                pyglet.graphics.draw(2, pyglet.gl.GL_LINES,
                                     ('v2f', (p1.x, p1.y, p2.x, p2.y)),
                                      ('c3f', (0.0, 1.0, 1.0, 0, 1, 1)))

        pyglet.gl.glPopMatrix()
        pyglet.gl.glLineWidth(1)


    def on_enter( self ):
        super( MainScene, self ).on_enter()
        self.do( Repeat( CallFunc( self.update ) ) )


### todo: put somewhere
circle_gllist = None
def draw_circle(point, radius, color = (1,0,0), steps = 10, filled = False):
    p = Vec2d(point)
    r = radius

    # global filled_circle_vertex_list
    # if filled_circle_vertex_list is not None and False:
        # glNewList(self.gllist,GL_COMPILE)
        # pyglet.gl.glPushMatrix()
        # glTranslatef(point.x, point.y, 1)
        # filled_circle_vertex_list.draw( pyglet.gl.GL_TRIANGLE_FAN )
        # pyglet.gl.glPopMatrix()
        # return

    global circle_gllist
    if circle_gllist is not None:
        pyglet.gl.glPushMatrix()
        pyglet.gl.glTranslatef(point.x, point.y, 1)
        pyglet.gl.glColor3f(color[0],color[1],color[2])
        pyglet.gl.glCallList(circle_gllist)
        pyglet.gl.glPopMatrix()
        return
    circle_gllist = pyglet.gl.glGenLists(1)

    pyglet.gl.glNewList(circle_gllist,pyglet.gl.GL_COMPILE)

    vs = []
    for r in range(steps):
        a = math.pi*2*r/float(steps)
        vs += [math.sin(a) * radius, math.cos(a) * radius]

    if filled:
        vs = vs[:2] + vs + vs[:-2]
        cs = color * (len(vs)/2)
        #filled_circle_vertex_list = pyglet.graphics.vertex_list(len(vs)/2, ('v2f', vs),
        #                      ('c4f', cs))
        pyglet.graphics.draw(len(vs)/2, pyglet.gl.GL_TRIANGLE_FAN,
                              ('v2f', vs),
                              ('c3f', cs))
    else:
        vs = vs[:2] + vs + vs[:-2]
        cs = color * (len(vs)/2)
        pyglet.graphics.draw(len(vs)/2, pyglet.gl.GL_LINE_STRIP,
                              ('v2f', vs),
                              ('c3f', cs))
    pyglet.gl.glEndList()

    return
